"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  INFINITE_PREFIX: () => INFINITE_PREFIX,
  KEY_PREFIX: () => KEY_PREFIX,
  KEY_SEPARATOR: () => KEY_SEPARATOR,
  POSTGREST_FILTER_KEY_PREFIX: () => POSTGREST_FILTER_KEY_PREFIX,
  createKeyGetter: () => createKeyGetter,
  decode: () => decode,
  encode: () => encode,
  getTable: () => getTable,
  infiniteMiddleware: () => infiniteMiddleware,
  isV1Response: () => isV1Response,
  middleware: () => middleware,
  useCountedPagination: () => useCountedPagination,
  useDeleteMutation: () => useDeleteMutation,
  useInfiniteQuery: () => useInfiniteQuery,
  useInfiniteScrollQuery: () => useInfiniteScrollQuery,
  useInsertMutation: () => useInsertMutation,
  usePaginationQuery: () => usePaginationQuery,
  usePostgrestFilterCache: () => usePostgrestFilterCache,
  useQuery: () => useQuery,
  useSubscription: () => useSubscription,
  useSubscriptionQuery: () => useSubscriptionQuery,
  useUpdateMutation: () => useUpdateMutation,
  useUpsertMutation: () => useUpsertMutation
});
module.exports = __toCommonJS(src_exports);

// src/lib/constants.ts
var KEY_PREFIX = "postgrest";
var POSTGREST_FILTER_KEY_PREFIX = "postgrest-filter";
var KEY_SEPARATOR = "$";
var INFINITE_PREFIX = "$inf$";

// src/lib/create-key-getter.ts
var createKeyGetter = (query, pageSize) => {
  if (!query)
    return () => null;
  return (pageIndex, previousPageData) => {
    if (previousPageData && !previousPageData.length)
      return null;
    const cursor = pageIndex * pageSize;
    return query.range(cursor, cursor + pageSize - 1);
  };
};

// src/lib/decode.ts
var decode = (key) => {
  if (typeof key !== "string")
    return null;
  const isInfinite = key.startsWith(INFINITE_PREFIX);
  let parsedKey = key.replace(INFINITE_PREFIX, "");
  const isPostgrestKey = parsedKey.startsWith(`${KEY_PREFIX}${KEY_SEPARATOR}`);
  if (!isPostgrestKey) {
    return null;
  }
  parsedKey = parsedKey.replace(`${KEY_PREFIX}${KEY_SEPARATOR}`, "");
  const [schema, table, queryKey, bodyKey, count, head] = parsedKey.split(KEY_SEPARATOR);
  const params = new URLSearchParams(queryKey);
  const limit = params.get("limit");
  const offset = params.get("offset");
  const countValue = count.replace("count=", "");
  return {
    limit: limit ? Number(limit) : void 0,
    offset: offset ? Number(offset) : void 0,
    bodyKey,
    count: countValue === "null" ? null : countValue,
    isHead: head === "head=true",
    isInfinite,
    key,
    queryKey,
    schema,
    table
  };
};

// src/lib/encode.ts
var import_postgrest_shared = require("@supabase-cache-helpers/postgrest-shared");
var encode = (parser) => {
  return [
    KEY_PREFIX,
    parser.schema ?? import_postgrest_shared.DEFAULT_SCHEMA_NAME,
    parser.table,
    parser.queryKey,
    parser.bodyKey ?? "null",
    `count=${parser.count}`,
    `head=${parser.isHead}`
  ].join(KEY_SEPARATOR);
};

// src/lib/get-table.ts
var getTable = (query) => query.url.pathname.split("/").pop();

// src/lib/middleware.ts
var import_postgrest_filter = require("@supabase-cache-helpers/postgrest-filter");
var middleware = (useSWRNext) => {
  return (key, fetcher, config) => {
    const query = key;
    if (!fetcher)
      throw new Error("No fetcher provided");
    return useSWRNext(
      encode(new import_postgrest_filter.PostgrestParser(query)),
      () => fetcher(query),
      config
    );
  };
};
var infiniteMiddleware = (useSWRInfiniteNext) => {
  return (keyFn, fetcher, config) => {
    return useSWRInfiniteNext(
      (index, previousPageData) => {
        const query = keyFn(index, previousPageData);
        if (!query)
          return null;
        return encode(
          new import_postgrest_filter.PostgrestParser(query)
        );
      },
      typeof fetcher === "function" ? (query) => fetcher(query) : fetcher,
      config
    );
  };
};

// src/lib/use-postgrest-filter-cache.ts
var import_swr = require("swr");
var import_postgrest_filter2 = require("@supabase-cache-helpers/postgrest-filter");
var usePostgrestFilterCache = () => {
  const { cache } = (0, import_swr.useSWRConfig)();
  return (query, opts) => {
    const key = [
      POSTGREST_FILTER_KEY_PREFIX,
      query,
      opts ? (0, import_postgrest_filter2.encodeObject)(opts) : null
    ].filter(Boolean).join(KEY_SEPARATOR);
    const cacheData = cache.get(key);
    if (cacheData && cacheData.data instanceof import_postgrest_filter2.PostgrestFilter) {
      return cacheData.data;
    }
    const filter = import_postgrest_filter2.PostgrestFilter.fromQuery(query, opts);
    cache.set(key, { data: filter });
    return filter;
  };
};

// src/mutate/use-delete-mutation.ts
var import_swr2 = require("swr");
var import_use_mutation = __toESM(require("use-mutation"));
var import_postgrest_fetcher = require("@supabase-cache-helpers/postgrest-fetcher");
var import_postgrest_mutate = require("@supabase-cache-helpers/postgrest-mutate");
function useDeleteMutation(qb, primaryKeys, query, opts) {
  const { mutate, cache } = (0, import_swr2.useSWRConfig)();
  const getPostgrestFilter = usePostgrestFilterCache();
  return (0, import_use_mutation.default)(
    (0, import_postgrest_fetcher.buildDeleteFetcher)(qb, primaryKeys, query),
    {
      ...opts,
      async onSuccess(params) {
        await (0, import_postgrest_mutate.deleteItem)(
          {
            input: params.data,
            primaryKeys,
            table: getTable(qb),
            schema: qb.schema,
            opts
          },
          {
            cacheKeys: Array.from(cache.keys()),
            getPostgrestFilter,
            mutate,
            decode
          }
        );
        if (opts == null ? void 0 : opts.onSuccess)
          await opts.onSuccess(params);
      }
    }
  );
}

// src/mutate/use-insert-mutation.ts
var import_use_mutation2 = __toESM(require("use-mutation"));
var import_swr3 = require("swr");
var import_postgrest_fetcher2 = require("@supabase-cache-helpers/postgrest-fetcher");
var import_postgrest_mutate2 = require("@supabase-cache-helpers/postgrest-mutate");
function useInsertMutation(qb, mode, primaryKeys, query, opts) {
  const { mutate, cache } = (0, import_swr3.useSWRConfig)();
  const getPostgrestFilter = usePostgrestFilterCache();
  return (0, import_use_mutation2.default)(
    (0, import_postgrest_fetcher2.buildInsertFetcher)(qb, mode, query),
    {
      ...opts,
      async onSuccess(params) {
        const result = !Array.isArray(params.data) ? [params.data] : params.data;
        await Promise.all(
          result.map(
            async (r) => await (0, import_postgrest_mutate2.upsertItem)(
              {
                input: r,
                primaryKeys,
                table: getTable(qb),
                schema: qb.schema,
                opts
              },
              {
                cacheKeys: Array.from(cache.keys()),
                getPostgrestFilter,
                mutate,
                decode
              }
            )
          )
        );
        if (opts == null ? void 0 : opts.onSuccess)
          await opts.onSuccess(params);
      }
    }
  );
}

// src/mutate/use-update-mutation.ts
var import_use_mutation3 = __toESM(require("use-mutation"));
var import_swr4 = require("swr");
var import_postgrest_fetcher3 = require("@supabase-cache-helpers/postgrest-fetcher");
var import_postgrest_mutate3 = require("@supabase-cache-helpers/postgrest-mutate");
function useUpdateMutation(qb, primaryKeys, query, opts) {
  const { mutate, cache } = (0, import_swr4.useSWRConfig)();
  const getPostgrestFilter = usePostgrestFilterCache();
  return (0, import_use_mutation3.default)(
    (0, import_postgrest_fetcher3.buildUpdateFetcher)(qb, primaryKeys, query),
    {
      ...opts,
      async onSuccess(params) {
        await (0, import_postgrest_mutate3.upsertItem)(
          {
            input: params.data,
            primaryKeys,
            table: getTable(qb),
            schema: qb.schema,
            opts
          },
          {
            cacheKeys: Array.from(cache.keys()),
            getPostgrestFilter,
            mutate,
            decode
          }
        );
        if (opts == null ? void 0 : opts.onSuccess)
          await opts.onSuccess(params);
      }
    }
  );
}

// src/mutate/use-upsert-mutation.ts
var import_use_mutation4 = __toESM(require("use-mutation"));
var import_swr5 = require("swr");
var import_postgrest_fetcher4 = require("@supabase-cache-helpers/postgrest-fetcher");
var import_postgrest_mutate4 = require("@supabase-cache-helpers/postgrest-mutate");
function useUpsertMutation(qb, mode, primaryKeys, query, opts) {
  const { mutate, cache } = (0, import_swr5.useSWRConfig)();
  const getPostgrestFilter = usePostgrestFilterCache();
  return (0, import_use_mutation4.default)(
    (0, import_postgrest_fetcher4.buildUpsertFetcher)(qb, mode, query),
    {
      ...opts,
      async onSuccess(params) {
        const result = !Array.isArray(params.data) ? [params.data] : params.data;
        await Promise.all(
          result.map(
            async (r) => await (0, import_postgrest_mutate4.upsertItem)(
              {
                input: r,
                primaryKeys,
                table: getTable(qb),
                schema: qb.schema,
                opts
              },
              {
                cacheKeys: Array.from(cache.keys()),
                getPostgrestFilter,
                mutate,
                decode
              }
            )
          )
        );
        if (opts == null ? void 0 : opts.onSuccess)
          await opts.onSuccess(params);
      }
    }
  );
}

// src/query/use-counted-pagination.ts
var import_react = require("react");
var useCountedPagination = ({
  pageSize,
  countQuery,
  dataQuery
}) => {
  const {
    count,
    error: countError,
    isValidating: isValidatingCount
  } = countQuery;
  const {
    data,
    error: dataError,
    isValidating: isValidatingData,
    setSize,
    size
  } = dataQuery;
  const [currentPageIndex, setCurrentPageIndex] = (0, import_react.useState)(0);
  const pageCount = (0, import_react.useMemo)(() => {
    if (!count || !pageSize)
      return null;
    return Math.ceil(count / pageSize);
  }, [count, pageSize]);
  return {
    pages: data,
    pageCount,
    setPage: async (idx) => {
      if (idx >= size) {
        await setSize((size2) => size2 + 1);
      }
      setCurrentPageIndex(idx);
    },
    pageIndex: currentPageIndex,
    currentPage: data ? data[currentPageIndex] ?? [] : [],
    nextPage: pageCount && currentPageIndex < pageCount - 1 ? async () => {
      if (currentPageIndex === size - 1) {
        await setSize((size2) => size2 + 1);
      }
      setCurrentPageIndex((page) => page + 1);
    } : null,
    previousPage: currentPageIndex > 0 ? () => setCurrentPageIndex((current) => current - 1) : null,
    isValidating: isValidatingCount || isValidatingData,
    error: countError ?? dataError
  };
};

// src/query/use-infinite-query.ts
var import_infinite = __toESM(require("swr/infinite"));
var import_postgrest_fetcher5 = require("@supabase-cache-helpers/postgrest-fetcher");
function useInfiniteQuery(query, config) {
  return (0, import_infinite.default)(
    createKeyGetter(query, (config == null ? void 0 : config.pageSize) ?? 20),
    (0, import_postgrest_fetcher5.createPaginationFetcher)(
      query,
      (key) => {
        const decodedKey = decode(key);
        if (!decodedKey) {
          throw new Error("Not an SWRPostgrest key");
        }
        return {
          limit: decodedKey.limit,
          offset: decodedKey.offset
        };
      },
      (config == null ? void 0 : config.pageSize) ?? 20
    ),
    {
      ...config,
      use: [
        ...(config == null ? void 0 : config.use) ?? [],
        infiniteMiddleware
      ]
    }
  );
}

// src/query/use-infinite-scroll-query.ts
var import_infinite2 = __toESM(require("swr/infinite"));
var import_react2 = require("react");
var import_lodash = require("lodash");
var import_postgrest_fetcher6 = require("@supabase-cache-helpers/postgrest-fetcher");
function useInfiniteScrollQuery(query, config) {
  const { data, error, isValidating, size, setSize } = (0, import_infinite2.default)(
    createKeyGetter(query, (config == null ? void 0 : config.pageSize) ?? 20),
    (0, import_postgrest_fetcher6.createPaginationHasMoreFetcher)(
      query,
      (key) => {
        const decodedKey = decode(key);
        if (!decodedKey) {
          throw new Error("Not an SWRPostgrest key");
        }
        return {
          limit: decodedKey.limit,
          offset: decodedKey.offset
        };
      },
      (config == null ? void 0 : config.pageSize) ?? 20
    ),
    {
      ...config,
      use: [
        ...(config == null ? void 0 : config.use) ?? [],
        infiniteMiddleware
      ]
    }
  );
  const { data: parsedData, hasMore } = (0, import_react2.useMemo)(() => {
    if (!Array.isArray(data))
      return { data, hasMore: false };
    const newData = (0, import_lodash.cloneDeep)(data);
    const lastPage = newData[newData.length - 1];
    const lastEntry = lastPage[lastPage.length - 1];
    return {
      data: newData.flatMap((page) => {
        var _a;
        if ((_a = page[page.length - 1]) == null ? void 0 : _a.hasMore) {
          page.pop();
        }
        return page;
      }),
      hasMore: (lastEntry == null ? void 0 : lastEntry.hasMore) ?? false
    };
  }, [data]);
  return {
    data: parsedData,
    loadMore: hasMore ? () => setSize(size + 1) : null,
    error,
    isValidating
  };
}

// src/query/use-pagination-query.ts
var import_infinite3 = __toESM(require("swr/infinite"));
var import_react3 = require("react");
var import_lodash2 = require("lodash");
var import_postgrest_fetcher7 = require("@supabase-cache-helpers/postgrest-fetcher");
function usePaginationQuery(query, config) {
  const { data, error, isValidating, size, setSize } = (0, import_infinite3.default)(
    createKeyGetter(query, (config == null ? void 0 : config.pageSize) ?? 20),
    (0, import_postgrest_fetcher7.createPaginationHasMoreFetcher)(
      query,
      (key) => {
        const decodedKey = decode(key);
        if (!decodedKey) {
          throw new Error("Not an SWRPostgrest key");
        }
        return {
          limit: decodedKey.limit,
          offset: decodedKey.offset
        };
      },
      (config == null ? void 0 : config.pageSize) ?? 20
    ),
    {
      ...config,
      use: [
        ...(config == null ? void 0 : config.use) ?? [],
        infiniteMiddleware
      ]
    }
  );
  const [currentPageIndex, setCurrentPageIndex] = (0, import_react3.useState)(0);
  const { data: parsedData, hasMore } = (0, import_react3.useMemo)(() => {
    if (!Array.isArray(data))
      return { data, hasMore: false };
    const newData = (0, import_lodash2.cloneDeep)(data);
    const lastPage = newData[newData.length - 1];
    const lastEntry = lastPage[lastPage.length - 1];
    return {
      data: newData.map((page) => {
        var _a;
        if ((_a = page[page.length - 1]) == null ? void 0 : _a.hasMore) {
          page.pop();
        }
        return page;
      }),
      hasMore: (lastEntry == null ? void 0 : lastEntry.hasMore) ?? false
    };
  }, [data]);
  return {
    pages: parsedData,
    currentPage: parsedData ? parsedData[currentPageIndex] ?? [] : [],
    pageIndex: currentPageIndex,
    setPage: (idx) => setCurrentPageIndex(idx),
    nextPage: hasMore || currentPageIndex < size - 1 ? () => {
      if (currentPageIndex === size - 1) {
        setSize((size2) => size2 + 1);
      }
      setCurrentPageIndex((page) => page + 1);
    } : null,
    previousPage: currentPageIndex > 0 ? () => setCurrentPageIndex((current) => current - 1) : null,
    error,
    isValidating
  };
}

// src/query/use-query.ts
var import_swr6 = __toESM(require("swr"));
var import_postgrest_fetcher8 = require("@supabase-cache-helpers/postgrest-fetcher");
function useQuery(query, mode, config) {
  const { data: res, ...rest } = (0, import_swr6.default)(
    query,
    (0, import_postgrest_fetcher8.createFetcher)(mode),
    {
      ...config,
      use: [...(config == null ? void 0 : config.use) ?? [], middleware]
    }
  );
  if (!res) {
    return { data: res, ...rest };
  }
  const { data, count } = res;
  return { data, count, ...rest };
}

// src/subscribe/types.ts
var isV1Response = (payload) => Boolean(payload.record);

// src/subscribe/use-subscription-query.ts
var import_swr7 = require("swr");
var import_react4 = require("react");
var import_supabase_js = require("@supabase/supabase-js");
var import_postgrest_mutate5 = require("@supabase-cache-helpers/postgrest-mutate");
function useSubscriptionQuery(client, channelName, filter, query, primaryKeys, opts) {
  const { mutate, cache } = (0, import_swr7.useSWRConfig)();
  const getPostgrestFilter = usePostgrestFilterCache();
  const [status, setStatus] = (0, import_react4.useState)();
  const [channel, setChannel] = (0, import_react4.useState)();
  (0, import_react4.useEffect)(() => {
    if (!client)
      return;
    const c = client.channel(channelName).on(
      import_supabase_js.REALTIME_LISTEN_TYPES.POSTGRES_CHANGES,
      filter,
      async (payload) => {
        let eventType = payload.eventType;
        let newRecord = payload.new;
        let oldRecord = payload.old;
        if (isV1Response(payload)) {
          eventType = payload.type;
          newRecord = payload.record;
          oldRecord = payload.old_record;
        }
        let data = newRecord ?? oldRecord;
        if (eventType !== import_supabase_js.REALTIME_POSTGRES_CHANGES_LISTEN_EVENT.DELETE) {
          const qb = client.from(payload.table).select(query);
          for (const pk of primaryKeys) {
            qb.eq(pk.toString(), data[pk]);
          }
          const res = await qb.single();
          if (res.data)
            data = res.data;
        }
        if (eventType === import_supabase_js.REALTIME_POSTGRES_CHANGES_LISTEN_EVENT.INSERT || eventType === import_supabase_js.REALTIME_POSTGRES_CHANGES_LISTEN_EVENT.UPDATE) {
          await (0, import_postgrest_mutate5.upsertItem)(
            {
              primaryKeys,
              input: data,
              table: payload.table,
              schema: payload.schema,
              opts
            },
            {
              cacheKeys: Array.from(cache.keys()),
              decode,
              getPostgrestFilter,
              mutate
            }
          );
        } else if (eventType === import_supabase_js.REALTIME_POSTGRES_CHANGES_LISTEN_EVENT.DELETE) {
          await (0, import_postgrest_mutate5.deleteItem)(
            {
              primaryKeys,
              input: data,
              table: payload.table,
              schema: payload.schema,
              opts
            },
            {
              cacheKeys: Array.from(cache.keys()),
              decode,
              getPostgrestFilter,
              mutate
            }
          );
        }
        if (opts == null ? void 0 : opts.callback) {
          opts.callback({
            ...payload,
            data,
            new: newRecord,
            old: oldRecord,
            eventType
          });
        }
      }
    ).subscribe((status2) => setStatus(status2));
    setChannel(c);
    return () => {
      if (c)
        c.unsubscribe();
    };
  }, []);
  return { channel, status };
}

// src/subscribe/use-subscription.ts
var import_swr8 = require("swr");
var import_react5 = require("react");
var import_supabase_js2 = require("@supabase/supabase-js");
var import_postgrest_mutate6 = require("@supabase-cache-helpers/postgrest-mutate");
function useSubscription(channel, filter, primaryKeys, opts) {
  const { mutate, cache } = (0, import_swr8.useSWRConfig)();
  const getPostgrestFilter = usePostgrestFilterCache();
  const [status, setStatus] = (0, import_react5.useState)();
  (0, import_react5.useEffect)(() => {
    if (!channel)
      return;
    const c = channel.on(
      import_supabase_js2.REALTIME_LISTEN_TYPES.POSTGRES_CHANGES,
      filter,
      async (payload) => {
        let eventType = payload.eventType;
        let newRecord = payload.new;
        let oldRecord = payload.old;
        if (isV1Response(payload)) {
          eventType = payload.type;
          newRecord = payload.record;
          oldRecord = payload.old_record;
        }
        if (eventType === import_supabase_js2.REALTIME_POSTGRES_CHANGES_LISTEN_EVENT.INSERT || eventType === import_supabase_js2.REALTIME_POSTGRES_CHANGES_LISTEN_EVENT.UPDATE) {
          await (0, import_postgrest_mutate6.upsertItem)(
            {
              primaryKeys,
              input: newRecord,
              table: payload.table,
              schema: payload.schema,
              opts
            },
            {
              cacheKeys: Array.from(cache.keys()),
              decode,
              getPostgrestFilter,
              mutate
            }
          );
        } else if (eventType === import_supabase_js2.REALTIME_POSTGRES_CHANGES_LISTEN_EVENT.DELETE) {
          await (0, import_postgrest_mutate6.deleteItem)(
            {
              primaryKeys,
              input: oldRecord,
              table: payload.table,
              schema: payload.schema,
              opts
            },
            {
              cacheKeys: Array.from(cache.keys()),
              decode,
              getPostgrestFilter,
              mutate
            }
          );
        }
        if (opts == null ? void 0 : opts.callback) {
          opts.callback({
            ...payload,
            new: newRecord,
            old: oldRecord,
            eventType
          });
        }
      }
    ).subscribe((status2) => setStatus(status2));
    return () => {
      if (c)
        c.unsubscribe();
    };
  }, []);
  return { status };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  INFINITE_PREFIX,
  KEY_PREFIX,
  KEY_SEPARATOR,
  POSTGREST_FILTER_KEY_PREFIX,
  createKeyGetter,
  decode,
  encode,
  getTable,
  infiniteMiddleware,
  isV1Response,
  middleware,
  useCountedPagination,
  useDeleteMutation,
  useInfiniteQuery,
  useInfiniteScrollQuery,
  useInsertMutation,
  usePaginationQuery,
  usePostgrestFilterCache,
  useQuery,
  useSubscription,
  useSubscriptionQuery,
  useUpdateMutation,
  useUpsertMutation
});
//# sourceMappingURL=index.js.map